/* Sieve of Eratosthenes, from C* Reference Manual, p.35 ff. */

#include <stdio.hs>      /* Parallel standard i-o header */
#include "cmtimer.hs"    /* Timing facility */

#define TRUE  1
#define FALSE 0
#define NPROCS 8000

domain sieve { int is_prime, number; };

domain sieve sieve_procs[NPROCS];

int LIMIT;

/* Find all primes <= LIMIT */

void sieve::find_primes()
 { mono int next_prime;
     int is_candidate;     /* poly by default */
   is_prime = FALSE;
   is_candidate = (number >= 2) & (number <= LIMIT);
   while (is_candidate)
    { next_prime = (<?= number);   /* minimum candidate */
      sieve_procs[next_prime-1].is_prime = TRUE;
      if ( (number % next_prime) == 0 ) is_candidate = FALSE;
      if ( next_prime*next_prime > LIMIT )
         { is_prime = TRUE;  is_candidate = FALSE; };
    };
 }

/* Print (some of) the primes, 10 per line
   pall:  -1 - none,  0 - in last 100 entries,  1 - all */

void sieve::print_primes(pall)
 mono int pall;
 { mono int j = 1;
   mono int nx;
    int pprime;
    pprime = (pall >= 0) & ((pall==0) ? number >= LIMIT-100 : 1) & is_prime;
    while (pprime)
     { nx = (<?= number);
       sieve_procs[nx-1].pprime = FALSE;
       printf("%6d ", nx);
       if ( (j++) == 10 ) { printf("\n");  j = 1; };
     };
    if (j != 1) printf("\n");
 }


/* Main program.  Commands are:
   d <int>  - display status: -1 just time, 0 in last 100, 1 all
   l <int>  - set limit to int
   p        - find primes
   q        _ quit           */

void main()
 { int display;  char c[2];
    display = 1;  LIMIT = 100;
    while ( printf("sieve>"), scanf("%1s", c), c[0] != 'q' )
     switch (c[0])
      { case 'd': scanf("%d", &display); break;
        case 'l': scanf("%d", &LIMIT);  break;
        case 'p':
           printf("Calculating primes ...\n");
	   CM_start_timer(1);
           [domain sieve].{ number = 1 + (this - &sieve_procs[0]);
                            find_primes();
                            CM_stop_timer(1);  CM_reset_timer();
                            print_primes(display);
                          };
	   break;
	default:   printf("Bad input - ignored\n"); break;
      }
 }